<?php
/**
 * @copyright Copyright (c) 2017 Joas Schilling <coding@schilljs.com>
 *
 * @author Joas Schilling <coding@schilljs.com>
 * @author Julius Haertl <jus@bitgrid.net>
 *
 * @license EUPL, Version 1.2 or later (the "License")
 *
 * Licensed under the EUPL, Version 1.2 or later (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

namespace OCA\Liiibre\Controller;

use OCP\App\IAppManager;
use OCP\AppFramework\Controller;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\ContentSecurityPolicy;
use OCP\AppFramework\Http\FileDisplayResponse;
use OCP\AppFramework\Utility\ITimeFactory;
use OCP\Files\IAppData;
use OCP\Files\NotFoundException;
use OCP\Files\SimpleFS\ISimpleFile;
use OCP\Files\SimpleFS\ISimpleFolder;
use OCP\IRequest;

class IconController extends Controller {
	private IAppData $appData;
	private IAppManager $appManager;
	private ITimeFactory $timeFactory;

	public function __construct(
		string $appName,
		IRequest $request,
		IAppData $appData,
		IAppManager $appManager,
		ITimeFactory $timeFactory
	) {
		parent::__construct($appName, $request);

		$this->appData = $appData;
		$this->appManager = $appManager;
		$this->timeFactory = $timeFactory;
	}

	/**
	 * @NoAdminRequired
	 * @NoCSRFRequired
	 */
	public function showIcon(string $icon): FileDisplayResponse {
		$folder = $this->appData->getFolder('icons');
		try {
			$iconFile = $folder->getFile($icon);
		} catch (NotFoundException $exception) {
			$iconFile = $this->getDefaultIcon($folder, 'external.svg');
		}

		if (strpos($icon, '-dark.') === false && $this->request->isUserAgent([
				IRequest::USER_AGENT_CLIENT_ANDROID,
				IRequest::USER_AGENT_CLIENT_IOS,
				IRequest::USER_AGENT_CLIENT_DESKTOP,
			])) {
			// Check if there is a dark icon as well
			$basename = pathinfo($iconFile->getName(), PATHINFO_FILENAME);
			$basename .= '-dark.';
			$basename .= pathinfo($iconFile->getName(), PATHINFO_EXTENSION);

			try {
				$iconFile = $folder->getFile($basename);
			} catch (NotFoundException $exception) {
			}
		}

		$response = new FileDisplayResponse($iconFile, Http::STATUS_OK, ['Content-Type' => $iconFile->getMimeType()]);
		$response->cacheFor(86400);
		$expires = new \DateTime();
		$expires->setTimestamp($this->timeFactory->getTime());
		$expires->add(new \DateInterval('PT24H'));
		$response->addHeader('Expires', $expires->format(\DateTime::RFC2822));
		$response->addHeader('Pragma', 'cache');
		$csp = new ContentSecurityPolicy();
		$response->setContentSecurityPolicy($csp);
		return $response;
	}

	protected function getDefaultIcon(ISimpleFolder $folder, string $file): ISimpleFile {
		try {
			return $folder->getFile($file);
		} catch (NotFoundException $exception) {
		}

		// Default icon is missing, copy it from img/
		$content = file_get_contents($this->appManager->getAppPath('liiibre') . '/img/' . $file);
		if ($content === false) {
			throw new NotFoundException();
		}

		$LiiibreSVG = $folder->newFile($file);
		$LiiibreSVG->putContent($content);
		return $LiiibreSVG;
	}
}
