<?php
/**
 * @copyright Copyright (c) 2017 Joas Schilling <coding@schilljs.com>
 *
 * @license EUPL, Version 1.2 or later (the "License")
 *
 * Licensed under the EUPL, Version 1.2 or later (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

namespace OCA\Liiibre;

use OCA\Liiibre\AppInfo\Application;
use OCA\Liiibre\Exceptions\SiteNotFoundException;
use OCP\App\IAppManager;
use OCP\Files\IAppData;
use OCP\IConfig;
use OCP\IGroupManager;
use OCP\IRequest;
use OCP\IUser;
use OCP\IUserSession;
use OCP\INavigationManager;
use OCP\IURLGenerator;

class SitesManager {
	protected IRequest $request;
	protected IConfig $config;
	protected IAppManager $appManager;
	protected IGroupManager $groupManager;
	protected IUserSession $userSession;
	protected IAppData $appData;
	protected IURLGenerator $urlGenerator;

	public function __construct(IRequest $request,
		IConfig $config,
		IAppManager $appManager,
		IGroupManager $groupManager,
		IUserSession $userSession,
		IAppData $appData,
		IURLGenerator $urlGenerator) {
		$this->request = $request;
		$this->config = $config;
		$this->appManager = $appManager;
		$this->groupManager = $groupManager;
		$this->userSession = $userSession;
		$this->appData = $appData;
		$this->urlGenerator = $urlGenerator;
	}

	public function getSiteById(int $id): array {
		$sites = $this->getSites();
		foreach ($sites as $index => $site) {
			if ($id == $site["id"]) {
				return $site;
			}
		}
		throw new SiteNotFoundException();
	}

	public function getSites(): array {
		$chat_url = $this->config->getAppValue(Application::APP_ID, "chat_url");
		$doc_url = $this->config->getAppValue(Application::APP_ID, "doc_url") ?: "https://doc.liiib.re";
		$doc_redirect = filter_var($this->config->getAppValue(Application::APP_ID, "doc_redirect"), FILTER_VALIDATE_BOOLEAN); 
		$sso_account_url = $this->config->getAppValue(Application::APP_ID, "sso_account_url");
		$sso_admin_url = $this->config->getAppValue(Application::APP_ID, "sso_admin_url");
		$sso_redirect = filter_var($this->config->getAppValue(Application::APP_ID, "sso_redirect"), FILTER_VALIDATE_BOOLEAN); 
		$visio_url = $this->config->getAppValue(Application::APP_ID, "visio_url");

		$user = $this->userSession->getUser();
		$isAdmin = false;
		if ($user instanceof IUser) {
			$isAdmin = $this->groupManager->isAdmin($user->getUID());
		}

		$sites = [];

		if ($chat_url) {
			$sites[] = [
				'id' => 1,
				'name' => "Chat",
				'icon' =>  "conversation.png",
				'url' => $chat_url,
				'type' => INavigationManager::TYPE_APPS,
				'redirect' => true,
			];
		}


		if ($sso_admin_url && $isAdmin) {
			$sites[] = [
				'id' => 2,
				'name' => "Admin SSO",
				'icon' =>  "users.png",
				'url' => $sso_admin_url,
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => $sso_redirect,
			];

			$sites[] = [
				'id' => 6,
				'name' => "Quota des comptes",
				'icon' =>  "quota.png",
				'url' => "/settings/users",
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => true,
			];
		}

		if ($sso_account_url) {
			$sites[] = [
				'id' => 3,
				'name' => "Mon compte Liiibre",
				'icon' =>  "compte.png",
				'url' => $sso_account_url,
				'type' => INavigationManager::TYPE_SETTINGS,
				'redirect' => $sso_redirect,
			];
		}

		$sites[] = [
			'id' => 4,
			'name' => "Centre de Documentation",
			'icon' =>  "lifesaver.png",
			'url' => "$doc_url",
			'type' => INavigationManager::TYPE_SETTINGS,
			'redirect' => $doc_redirect,
		];

		if ($visio_url) {
			$sites[] = [
				'id' => 5,
				'name' => "Visio",
				'icon' =>  "meet.png",
				'url' => $visio_url,
				'type' => INavigationManager::TYPE_APPS,
				'redirect' => true,
			];
		}

		return $sites;
	}

	public function getNavigationEntries(): array {
		$sites = $this->getSites();
		$entries = [];
		foreach ($sites as $id => $site) {
			$image = $this->urlGenerator->linkToRoute('liiibre.icon.showIcon', ['icon' => $site['icon']]);
			$href = $site['url'];
			if (!$site['redirect']) {
				$href = $this->urlGenerator->linkToRoute('liiibre.site.showPage', ['id' => $site['id']]);
			}

			$entries[] = [
				'id' => 'liiibre_index' . $site['id'],
				'order' => 80 + $site['id'],
				'href' => $href,
				'icon' => $image,
				'type' => $site['type'],
				'name' => $site['name'],
			];
		}

		return $entries;
	}
}
